/*
 *   this file constains all the sql statements used to create all base tables
 *   of system.definition_schema.
 *   
 *   it is obtained from sql3, chapter 20, definition_schema, with
 *   some limited modifications.
 *
 */
create database definition_schema;

create table definition_schema.users  (
     user_name                    varchar(20),
     password                     varchar(48),
     user_id                      int,
     date_created                 datetime,
     constraint users_primary_key primary key (user_name)
) engine=fpcbase  ;

/*
 *   
 */
create table definition_schema.schemata (
     catalog_name           varchar(48),
     schema_name            varchar(48),
     schema_owner           varchar(48) not null,
     schemata_id              int, 
     date_created           datetime,
     no_grids               int,
     list_of_grids          varchar(2048),
     constraint schemata_primary_key primary key (catalog_name,schema_name )
--     constraint schemata_foreign_key foreign key (schema_owner) references users
) engine=fpcbase  ;

create table definition_schema.tables
     (
     table_catalog          varchar(48),
     table_schema           varchar(48),
     table_name             varchar(48),
     table_type             varchar(48),
     table_degree           int,
     is_updatable           varchar(48),
     cache_size             int,
     global_id              int,

     constraint tables_primary_key
       primary key ( table_catalog, table_schema, table_name )
) engine=fpcbase ;

create table definition_schema.index_files
     (
     global_id              int,
     key_index              int,
     file_name              varchar(256),
     index_name             varchar(48),

     constraint tables_primary_key
       primary key ( global_id,key_index )
) engine=fpcbase ;
 

create table definition_schema.create_table_stmt
     (
     table_catalog          varchar(48),
     table_schema           varchar(48),
     table_name             varchar(48),
     table_stmt             varchar(2048),

     constraint create_table_stmt_primary_key
       primary key ( table_catalog, table_schema, table_name )
) engine=fpcbase ;


create table definition_schema.sequences (
	sequence_catalog varchar(48),
	sequence_schema  varchar(48),
	sequence_name    varchar(48),
        sequence_value   int not null,
        start_value      int default 1,
	increment_value  int default 1,
        min_value        int,
        max_value        int,
	is_cycle         char(1),
  	constraint sequence_primary_key
           primary key(sequence_catalog,sequence_schema,sequence_name)
) engine=fpcbase ;

/*
 *  The first three columns represent the name of the virtual table, and the 4th the partition name.
 *
 */
create table definition_schema.table_partitions  (
     table_catalog          varchar(48),
     table_schema           varchar(48),
     table_name             varchar(48),
     grid_order             int,
     range_order            int,
     column_order           int,

     table_partition        varchar(256),

     --  for the grid partition
     grid_column	    varchar(48),
     grid_condition         varchar(1024),
       
     -- for the range partition
     range_column	    int,
     range_value            varchar(48),

     -- for the column partition
     list_of_columns        varchar(1024),

     constraint table_partitions_primary_key
       primary key (table_catalog,table_schema,table_name,grid_order,range_order,column_order)
) engine=fpcbase ;

/*
 *  Store all the info about the grid partitions
 */
create table definition_schema.table_distribution  (
     table_catalog          varchar(48),
     table_schema           varchar(48),
     table_name             varchar(48),
     no_of_grids            int,
     grid_id                int not null,	
     list_of_grids          varchar(1024),
     grid_column	    varchar(48),
     expression             varchar(1024),
     second_parameter       int, 
     third_parameter        int,
     constraint table_partitions_primary_key
       primary key (table_catalog,table_schema,table_name)
) engine=fpcbase  ;



/*
 *  This table store the parent information for each partition, including
 *   (1) the virtual table with the same column structures as the partition, and
 *   (2) the virtual table with the same primary key structure as the partition
 */
create table definition_schema.partition_parent  (
     table_catalog          varchar(48),
     table_schema           varchar(48),
     table_partition        varchar(48),
     column_table           varchar(48),
     primary_key_table      varchar(48),
     constraint partition_parent_primary_key
         primary key (table_catalog,table_schema,table_partition)
) engine=fpcbase;


/*
 *  The following three tables are used to store the grid information
 *
 *   to store all the information about grid nodes
 */
create table definition_schema.grid_nodes (
     grid_id         int,
     ip_address      varchar(1024),
     grid_name       varchar(48),
     client_socket   int not null,
     status          int, 
     constraint grid_node_primary_key
       primary key (grid_id)
) engine=fpcbase  ;



/*
 *  Store all the configured values for node replications
 */
create table definition_schema.replications (
     grid_id                int,
     replication_id         int,
     ip_address             varchar(64) not null,	
     client_socket          int,  -- the main socket
     internal_socket        int,  -- for grid node communication
     status                 int,
     process_id             int,  
     constraint replication_primary_key primary key (grid_id,replication_id)
--     constraint replication_foreign_key foreign key (grid_id) references grid_nodes
) engine=fpcbase;

/*
 *  Store all the configured values for node replications
 */
create table definition_schema.directories (
     grid_id                int,
     replication_id         int,
     installation           varchar(1024) not null,
     data_directory         varchar(1024) not null,
     log_directory          varchar(1024) not null,	
     snapshot_directory     varchar(1024) not null,	
     lobs_directory         varchar(1024),	
     ind_directory          varchar(1024),	
     constraint replication_primary_key primary key (grid_id,replication_id)
--     constraint replication_foreign_key foreign key (grid_id) references grid_nodes
) engine=fpcbase;

/*
 *  Store the status history of all node replications
 */
create table definition_schema.status_history (
     grid_id                int,
     replication_id         int,
     time_changed           datetime,
     new_status             int,
     constraint replication_primary_key primary key (grid_id,replication_id,time_changed)
) engine=fpcbase;

/*
 *   to store all the information about grid sockets
 */
create table definition_schema.grid_sockets (
     source_grid_id         int,
     destination_grid_id    int,
     no_connections         int not null,
     destination_port       int not null,
     flags                  int,

     constraint grid_sockets_primary_key
       primary key ( source_grid_id, destination_grid_id )
/*
    ,
     constraint grid_sockets_foreign_key_sid
       foreign key (source_grid_id) references definition_schema.grid_nodes,

     constraint grid_sockets_foreign_key_did
       foreign key (destination_grid_id)
         references definition_schema.grid_nodes
*/
) engine=fpcbase  ;


/*
 *   to store all the information about grid sockets
 */
create table definition_schema.replication_sockets (
     grid_id                int,
     source_node_id         int,
     destination_node_id    int,
     no_connections         int not null,
     destination_port       int not null,
     flags                  int,
     constraint replicaiton_sockets_primary_key
       primary key ( grid_id,source_node_id, destination_node_id )
/*
    ,
     constraint replicaiton_node_foreign_key_did
       foreign key (grid_id,source_node_id)
         references definition_schema.replications,
	 
     constraint replicaiton_node2_foreign_key_did
       foreign key (grid_id,destination_node_id)
         references definition_schema.replications
*/
) engine=fpcbase  ;


/*
 *  this table is used to store the system limitation.
 *  the table will be cleared and then re-populated when the server starts,
 *  because its content largely depends on valued of the configuration variables
 */
create table definition_schema.system_limitation (
	item varchar(120),
	limitation int,
	constraint system_limitatin_primary_key
         primary key (item)
) engine=fpcbase  ;


/*
 *  this table is used to store the values of permanent configuration variables.
 *  by a permanent configuration variable, we mean the configuration variables
 *  whose values are determined in the creation of the initial database.
 *  
 *  datadirectory, lobdirectory,inddirectory, logdirectory
 *  lobschema, 
 *
 */
create table definition_schema.configuration_values (
	item  varchar(120),
	value varchar(128),
	constraint cv_primary_key
         primary key (item)
) engine=fpcbase;



create table definition_schema.sqlstate_message (
     sql_state  varchar(40),
     sql_status varchar(1),
     error_class varchar(64),
     description varchar(1024),
     sql_code   integer,
     constraint sqlstate_message_primary_key
       primary key ( sql_state )
) engine=fpcbase;

create table definition_schema.checkpoint_record (
       chk_time datetime primary key,
       clients int,
       transaction_id int,
       roll_backs int,
       committed int,
       transaction_per_minute int
) engine=fpcbase  ;


create table  definition_schema.table_constraints (
     constraint_catalog    varchar(64),
     constraint_schema     varchar(64),
     constraint_name       varchar(64),
     table_schema          varchar(64) not null,
     table_name            varchar(64),
     constraint_type       varchar(64) not null,
     constraint table_constraints_primary_key
       primary key (constraint_catalog,constraint_schema,table_name,constraint_name)
) engine=fpcbase;

create table definition_schema.key_column_usage  (
     constraint_catalog    varchar(64),
     constraint_schema     varchar(64),
     constraint_name       varchar(64),
     table_catalog         varchar(64) not null,
     table_schema          varchar(64)  not null,
     table_name            varchar(64)  not null,
     column_name           varchar(64)  not null,
     ordinal_position      int   not null,
     position_in_unique_constraint bigint,

     referenced_table_schema varchar(64),
     referenced_table_name   varchar(64),
     referenced_column_name  varchar(64),

     constraint key_column_usage_primary_key
     primary key (constraint_catalog,constraint_schema,table_name,constraint_name,
                  column_name )
) engine=fpcbase;

/*
 *
 */
CREATE TABLE definition_schema.referential_constraints (
    constraint_catalog        varchar(48),
    constraint_schema         varchar(48),
    constraint_name           varchar(48),
    unique_constraint_catalog varchar(48)   NOT NULL,
    unique_constraint_schema  varchar(48)   NOT NULL,
    unique_constraint_name    varchar(48)   NOT NULL,
    match_option              varchar(48),
    update_rule               varchar(48),
    delete_rule               varchar(48),
    table_name                varchar(48),
    referenced_table_name     varchar(48),
    constraint referential_constraints_primary_key
      primary key (constraint_catalog, constraint_schema,table_name,constraint_name )
) engine=fpcbase;

create table definition_schema.temp_tables (
     client_id              int,
     table_schema           varchar(48),
     table_name             varchar(48),
     temp_name              varchar(48),
     constraint temp_primary_key
       primary key (client_id,table_schema,table_name )
) engine=fpcbase ;

create table definition_schema.table_privileges (
     grantor                varchar(64),
     grantee                varchar(64),
     table_catalog          varchar(64),
     table_schema           varchar(64),
     table_name             varchar(64),
     privilege_type         varchar(32),
     is_grantable           varchar(16) not null,
     primary key (grantee,table_catalog,table_schema,table_name,privilege_type)
) engine=fpcbase;

/*
 *  Store all the info about the grid partitions
 */
create table definition_schema.procedures  (
     proc_schema           varchar(48),
     proc_name             varchar(48),
     param_list            varchar(2048),
     no_of_grids           int,
     list_of_grids         varchar(1024),
     partition_param       int,    -- the nth parameter for partition 
     expression            varchar(1024),
     second_param          int, 
     third_parame          int,
     primary key (proc_schema,proc_name)
) engine=fpcbase;


use definition_schema;

insert into definition_schema.table_privileges values ('root','root','*','*','*','all','yes');

insert into definition_schema.tables values ('def','definition_schema','users','table',4,'updatable',0,1);
insert into definition_schema.tables values ('def','definition_schema','tables','table',8,'updatable',0,2);
insert into definition_schema.tables values ('def','definition_schema','schemata','table',6,'updatable',0,3);
insert into definition_schema.tables values ('def','definition_schema','table_distribution','table',8,'updatable',0,4);
insert into definition_schema.tables values ('def','definition_schema','grid_nodes','table',4,'updatable',0,5);

insert into definition_schema.tables values ('def','definition_schema','replications','table',4,'updatable',0,6);
insert into definition_schema.tables values ('def','definition_schema','grid_sockets','table',5,'updatable',0,7);
insert into definition_schema.tables values ('def','definition_schema','replication_sockets','table',6,'updatable',0,8);
insert into definition_schema.tables values ('def','definition_schema','system_limitation','table',2,'updatable',0,9);
insert into definition_schema.tables values ('def','definition_schema','checkpoint_record','table',6,'updatable',0,10);

insert into definition_schema.tables values ('def','definition_schema','sqlstate_message','table',6,'updatable',0,11);
insert into definition_schema.tables values ('def','definition_schema','index_files','table',6,'updatable',0,12);
insert into definition_schema.tables values ('def','definition_schema','create_table_stmt','table',6,'updatable',0,13);
insert into definition_schema.tables values ('def','definition_schema','table_partitions','table',6,'updatable',0,14);
insert into definition_schema.tables values ('def','definition_schema','partition_parent','table',6,'updatable',0,15);

insert into definition_schema.tables values ('def','definition_schema','configuration_values','table',6,'updatable',0,16);
insert into definition_schema.tables values ('def','definition_schema','table_constraints','table',6,'updatable',0,17);
insert into definition_schema.tables values ('def','definition_schema','key_column_usage','table',6,'updatable',0,18);
insert into definition_schema.tables values ('def','definition_schema','referential_constraints','table',6,'updatable',0,19);
insert into definition_schema.tables values ('def','definition_schema','sequences','table',9,'updatable',0,20);
insert into definition_schema.tables values ('def','definition_schema','status_history','table',4,'updatable',0,21);
insert into definition_schema.tables values ('def','definition_schema','directories','table',8,'updatable',0,22);
insert into definition_schema.tables values ('def','definition_schema','temp_tables','table',3,'updatable',0,23);
insert into definition_schema.tables values ('def','definition_schema','table_privileges','table',7,'updatable',0,24);
insert into definition_schema.tables values ('def','definition_schema','procedures','table',13,'updatable',0,25);
